#' @importFrom Rcpp sourceCpp
#' @importFrom stats median rnorm sd
#' @importFrom graphics abline legend lines plot par
#' @import zoo
#' @import np
#' @import e1071
NULL

#' Monthly returns of the Standard and Poor's Composite index
#'
#' The monthly returns of the S&P Composite index and the risk-free rate of return
#' over the period from January 1857 to December 2015
#'
#' @name sp500.monthly
#' @usage sp500.monthly
#' @family data sets
#' @format An object of class \code{"zoo"} containing the following variables:
#' \describe{
#'   \item{TOT}{total return (capital gain plus dividend yield)}
#'   \item{CAP}{capital gain return}
#'   \item{RF}{risk-free rate of return}
#' }
NULL

#' Daily returns of the Standard and Poor's Composite index
#'
#' The daily returns of the S&P Composite index and the risk-free rate of return
#' over the period from July 1926 to December 2015
#'
#' @name sp500.daily
#' @usage sp500.daily
#' @family data sets
#' @format An object of class \code{"zoo"} containing the following variables:
#' \describe{
#'   \item{TOT}{total return (capital gain plus dividend yield)}
#'   \item{CAP}{capital gain return}
#'   \item{RF}{risk-free rate of return}
#' }
NULL

#' Monthly returns of the Dow Jones Industrial Average index
#'
#' The monthly returns of the DJIA index and the risk-free rate of return
#' over the period from July 1926 to December 2015
#'
#' @name djia.monthly
#' @usage djia.monthly
#' @family data sets
#' @format An object of class \code{"zoo"} containing the following variables:
#' \describe{
#'   \item{TOT}{total return (capital gain plus dividend yield)}
#'   \item{CAP}{capital gain return}
#'   \item{RF}{risk-free rate of return}
#' }
NULL

#' Daily returns of the Dow Jones Industrial Average index
#'
#' The daily returns of the DJIA index and the risk-free rate of return
#' over the period from July 1926 to December 2015
#'
#' @name djia.daily
#' @usage djia.daily
#' @family data sets
#' @format An object of class \code{"zoo"} containing the following variables:
#' \describe{
#'   \item{TOT}{total return (capital gain plus dividend yield)}
#'   \item{CAP}{capital gain return}
#'   \item{RF}{risk-free rate of return}
#' }
NULL

Lag <- function(price, k) {
  # construct a lagged version of the price
  n <- length(price)
  price.lag <- c(rep(price[1],k),price[1:(n-k)])
  return(price.lag)
}

#' Computes the Zero Lag Exponential Moving Average (ZMA)
#'
#' This function computes the ZMA of prices
#'
#' @param price the vector of prices
#' @param n the length of the averaging window (a scalar)
#' @return the ZMA of prices (a vector)
#' @usage ZMA(price, n)
#' @family moving average functions
#' @references
#' Ehlers J. F. and Way R. (2010). Zero Lag (Well, Almost), Technical Analysis of Stocks and Commodities, 28 (12), 30-35.
#' @export

ZMA <- function(price, n) {
  # computes the Zero Lag Exponential Moving Average
  # price - the vector of prices
  # k - the size of the averaging window
  lagtime <- floor((n-1)/2)
  price.lag <- Lag(price, lagtime)
  ema <- EMA(price,n)
  ema.lagged <- EMA(price.lag,n)
  zlema <- 2*ema - ema.lagged
  return(zlema)
}

#' Computes the Hull Moving Average (HMA)
#'
#' This function computes the HMA of prices
#'
#' @param price the vector of prices
#' @param n the length of the averaging window (a scalar)
#' @return the HMA of prices (a vector)
#' @usage HMA(price, n)
#' @family moving average functions
#' @references
#' Hull, A. (2005). How to Reduce Lag in a Moving Average, \url{http://www.alanhull.com/hull-moving-average}
#' @export

HMA <- function(price, n) {
  # first WMA
  m <- floor(n/2)
  if(m < 1) m <- 1
  wma <- LMA(price, m)
  # the difference between two LMA
  dwma <- 2*wma - LMA(price,n)
  # last WMA
  m <- floor(sqrt(n))
  if(m < 1) m <- 1
  hma <- LMA(dwma, m)
  return(hma)
}

#' Computes the Double Exponential Moving Average (DMA)
#'
#' This function computes the DMA of prices
#'
#' @param price the vector of prices
#' @param n the length of the averaging window (a scalar)
#' @return the DMA of prices (a vector)
#' @usage DMA(price, n)
#' @family moving average functions
#' @references
#' Mulloy P. G. (1994). Smoothing Data With Faster Moving Averages, Technical Analysis of Stocks and Commodities, 12 (1), 11-19.
#' @export

DMA <- function(price, n) {
  # computes the Double Exponential Moving Average
  # price - the vector of prices
  # k - the size of the averaging window
  ema <- EMA(price, n)
  dema <- 2*ema - EMA(ema, n)
  return(dema)
}

MAC <- function(price, fast, slow, FUN=EMA) {
  # computes the trading signal based on
  # the Double Crossover Method with short average of s and long average of k
  signal <- FUN(price, fast) - FUN(price, slow)
  return(signal)
}

MACD <- function(price, fast, slow, final, FUN=EMA) {
  # computes the trading signal based on
  # the MACD
  mac <- FUN(price, fast) - FUN(price, slow)
  signal <- mac - FUN(mac, final)
  return(signal)
}

CDIR <- function(price, winsize, FUN=SMA) {
  # computes the trading signal based on
  # the CDIR
  ma <- FUN(price, winsize)
  signal <- c(0,diff(ma))
  return(signal)
}

MAE <- function(price, winsize, percentage, FUN=SMA) {
  # computes the trading signal based on
  # the Moving Average Envelope
  ma <- FUN(price, winsize)
  #n <- length(price)
  upper <- ma*(1+percentage/100)
  lower <- ma*(1-percentage/100)
  signal <- mae_signal(price, lower, upper)
  return(signal)
}


#' Simulates the returns to the Moving Average Crossover (MAC) strategy
#'
#' This function simulates the returns to the MAC strategy
#'
#' @param totret the vector of total returns
#' @param rfret the vector of risk-free returns
#' @param dates the vector of dates
#' @param capret the vector of capital gain returns
#' @param tc the amount of one-way transaction costs (a scalar)
#' @param shorts a logical value indicating the type of the strategy when a Sell signal is generated.
#' If \code{shorts=FALSE}, money is just invested in cash; otherwise, the asset is sold short
#' @param fast a vector or a scalar that determines the size of the shorter averaging window
#' @param slow a vector or a scalar that determines the size of the longer averaging window
#' @param FUN a function that determines the type of a moving average
#' @return An object to be used by other functions. This object contains the returns to
#' the simulated strategies and the description of each strategy (the type of a moving average,
#' the sizes of the shorter and the longer windows)
#' @note The function \code{FUN} is either \code{\link{SMA}}, \code{\link{LMA}}, \code{\link{EMA}},
#' \code{\link{DMA}}, \code{\link{HMA}} or \code{\link{ZMA}}. The trading signal is computed using the vector of capital gain returns \code{capret}. If \code{capret=NULL},
#' then the trading signal is computed using the vector of total returns \code{totret}.
#' If one of the arguments \code{fast} or \code{slow} is a vector, then the function simulates a number
#' of trading strategies for all combinations of \code{fast} and \code{slow} such that \code{fast} \eqn{<} \code{slow}
#' @usage sim.mac.strategy(totret, rfret, dates, capret=NULL, tc=0, shorts = FALSE,
#'                         fast=50, slow=200, FUN=SMA)
#' @family strategy simulation functions
#' @examples
#' \dontrun{
#' data <- djia.monthly # use the monthly data for DJIA index
#' dates <- index(data)
#' data <- coredata(data)
#' capret <- data[,"CAP"]
#' totret <- data[,"TOT"]
#' rfret <- data[,"RF"]
#' tc <- 0.0025 # this defines 0.25% one-way transaction costs
#' fast <- 1:8  # define the sizes of the fast (shorter) averaging window
#' slow <- 2:15 # define the sizes of the slow (longer) averaging window
#' shorts <- FALSE
#' results <- sim.mac.strategy(totret=totret, rfret=rfret, dates=dates,
#'                             capret=capret, tc=tc, shorts=shorts, fast=fast, slow=slow, FUN=EMA)
#' }
#' @export sim.mac.strategy

sim.mac.strategy <- function(totret, rfret, dates, capret=NULL,
                             tc=0, shorts = FALSE, fast=50, slow=200, FUN=SMA) {

  # This function simulate the returns to
  # the MAC strategy for different sizes of fast and slow moving averages
  # Inputs:
  # totret - vector of total returns to the passive strategy
  # rfret  - vector of returns to the risk-free asset
  # capret - vector of capital returns to the passive strategy
  # dates  - vector of dates
  # tc     - amount of transaction costs
  # shorts - defines whether to sell short (if TRUE) or stay in cash
  # fast   - vector of sizes of the fast averaging window
  # slow   - vector of sizes of the slow averaging window
  # FUN    - function that computes the moving average
  # Remarks:
  # The trading singal is computed using "capret". If capret is missing,
  # then the trading signal is computed using "totret"
  # Function simulates the valid strategies where fast < slow
  # Output:
  # A list of objects

  # initial check for robustness
  if(!is.logical(shorts))
    stop("Argument 'shorts' must be of logical type!")
  nobs <- length(totret)
  if(length(rfret)!=nobs)
    stop("Mismatch in the number of observations between totret and rfret!")
  if(length(dates)!=nobs)
    stop("Mismatch in the number of observations between totret and dates!")
  if(length(capret)==0)
    capret <- totret
  else if(length(capret)!=nobs)
    stop("Mismatch in the number of observations between totret and capret!")
  fun.name <- deparse(substitute(FUN))
  if(!exists(fun.name, mode='function')) {
    err.str <- paste("Something is wrong. Function", fun.name, "cannot be found!")
    stop(err.str)
  }

  # remove elements with values < 1
  slow <- slow[which(slow > 1)]
  # remove elements with values < 0
  fast <- fast[which(fast > 0)]

  # compute the lengths
  n.slow <- length(slow)
  n.fast <- length(fast)
  if(n.slow==0) stop("Zero length of slow vector!")
  if(n.fast==0) stop("Zero length of fast vector!")

  # construct the initial set of parameters
  N <- n.slow*n.fast
  param <- matrix(NA,nrow=3,ncol=N)
  for(i in 1:n.fast)
    for(j in 1:n.slow) {
      ind <- (i-1)*n.slow + j
      param[1,ind] <- fast[i]
      param[2,ind] <- slow[j]
    }

  # delete the pairs of parameters where fast >= slow
  ind <- which(param[2,] > param[1,])
  if(length(ind)==1) param <- as.matrix(param[,ind],nrow=3,ncol=1) else param <- param[,ind]

  # construct the strategy description
  N <- ncol(param)
  fun.name <- rep(deparse(substitute(FUN)),N)
  rule.name <- rep("MAC",N)
  desc <- data.frame(Trading.Rule=rule.name, MA.Type=fun.name,
                    Param.1 = param[1,], Param.2 = param[2,], Param.3 = param[3,])

  # construct the matrix of active returns
  actret <- matrix(nrow=nobs, ncol=N)
  instocks <- matrix(FALSE, nrow=nobs, ncol=N)
  start <- rep(0,N) # vector of starting index of each active strategy
  price <- cumprod(1+capret) # construct the price index
  for(i in 1:N) {
    signal <- FUN(price, param[1,i]) - FUN(price, param[2,i]) # compute the trading signal
    signal <- signal > 0 # convert to boolean
    start[i] <- param[2,i] # current starting index
    actret[,i] <- sim_strategy(totret, rfret, signal, tc, shorts, start[i])
    instocks[,i] <- signal
  }

  # return a list with results
  result <- list(dates=dates, totret=totret, rfret=rfret, tc=tc, shorts=shorts,
                 start=start, actret=actret, desc=desc, instocks=instocks, resname="MAC")
  return(result)
}

#' Simulates the returns to the Moving Average Envelope (MAE) strategy
#'
#' This function simulates the returns to the MAE strategy
#'
#' @param totret the vector of total returns
#' @param rfret the vector of risk-free returns
#' @param dates the vector of dates
#' @param capret the vector of capital gain returns
#' @param tc the amount of one-way transaction costs (a scalar)
#' @param shorts a logical value indicating the type of the strategy when a Sell signal is generated.
#' If \code{shorts=FALSE}, money is just invested in cash; otherwise, the asset is sold short
#' @param winsize a vector or a scalar that determines the size of the averaging window
#' @param percentage a vector or a scalar that determines the envelope percentage
#' @param FUN a function that determines the type of a moving average
#' @return An object to be used by other functions. This object contains the returns to
#' the simulated strategies and the description of each strategy (the type of a moving average,
#' the sizes of the averaging windows, and the envelope percentages)
#' @note The function \code{FUN} is either \code{\link{SMA}}, \code{\link{LMA}}, \code{\link{EMA}},
#' \code{\link{DMA}}, \code{\link{HMA}} or \code{\link{ZMA}}. The trading signal is computed using the vector of capital gain returns \code{capret}. If \code{capret=NULL},
#' then the trading signal is computed using the vector of total returns \code{totret}.
#' @usage sim.mae.strategy(totret, rfret, dates, capret=NULL, tc=0, shorts = FALSE,
#'                         winsize=200, percentage=1, FUN=SMA)
#' @family strategy simulation functions
#' @examples
#' \dontrun{
#' data <- djia.monthly # use the monthly data for DJIA index
#' dates <- index(data)
#' data <- coredata(data)
#' capret <- data[,"CAP"]
#' totret <- data[,"TOT"]
#' rfret <- data[,"RF"]
#' tc <- 0.0025 # this defines 0.25% one-way transaction costs
#' winsize <- 2:20
#' percentage <- 0:10
#' shorts <- FALSE
#' results <- sim.mae.strategy(totret=totret, rfret=rfret, dates=dates,
#'                             capret=capret, tc=tc, winsize=winsize,
#'                             percentage=percentage, slow=slow, FUN=EMA)
#' }
#' @export sim.mae.strategy

sim.mae.strategy <- function(totret, rfret, dates, capret=NULL,
                             tc=0, shorts = FALSE, winsize=200, percentage=1, FUN=SMA) {

  # This function simulate the returns to the MAE strategy
  # Inputs:
  # totret - vector of total returns to the passive strategy
  # rfret  - vector of returns to the risk-free asset
  # capret - vector of capital returns to the passive strategy
  # dates  - vector of dates
  # tc     - amount of transaction costs
  # shorts - defines whether to sell short (if TRUE) or stay in cash
  # winsize - vector of sizes of the averaging window
  # percentages - vector of percentages to create envelope
  # FUN    - function that computes the moving average
  # Remarks:
  # The trading singal is computed using "capret". If capret is missing,
  # then the trading signal is computed using "totret"
  # Function simulates the valid strategies where fast < slow
  # Output:
  # A list of objects

  # initial check for robustness
  if(!is.logical(shorts))
    stop("Argument 'shorts' must be of logical type!")
  nobs <- length(totret)
  if(length(rfret)!=nobs)
    stop("Mismatch in the number of observations between totret and rfret!")
  if(length(dates)!=nobs)
    stop("Mismatch in the number of observations between totret and dates!")
  if(length(capret)==0)
    capret <- totret
  else if(length(capret)!=nobs)
    stop("Mismatch in the number of observations between totret and capret!")
  fun.name <- deparse(substitute(FUN))
  if(!exists(fun.name, mode='function')) {
    err.str <- paste("Something is wrong. Function", fun.name, "cannot be found!")
    stop(err.str)
  }

  # remove elements with winsize < 1
  winsize <- winsize[which(winsize > 1)]
  # remove elements with percentages < 0
  percentage <- percentage[which(percentage >= 0)]

  # compute the lengths
  n.winsize <- length(winsize)
  n.percentage <- length(percentage)
  if(n.winsize==0) stop("Zero length of winsize vector!")
  if(n.percentage==0) stop("Zero length of percentage vector!")

  # construct the set of parameters
  N <- n.winsize*n.percentage
  param <- matrix(NA,nrow=3,ncol=N)
  for(i in 1:n.winsize)
    for(j in 1:n.percentage) {
      ind <- (i-1)*n.percentage + j
      param[1,ind] <- winsize[i]
      param[2,ind] <- percentage[j]
    }

  # construct the strategy description
  N <- ncol(param)
  fun.name <- rep(deparse(substitute(FUN)),N)
  rule.name <- rep("MAE",N)
  desc <- data.frame(Trading.Rule=rule.name, MA.Type=fun.name,
                     Param.1 = param[1,], Param.2 = param[2,], Param.3 = param[3,])

  # construct the matrix of active returns
  actret <- matrix(nrow=nobs, ncol=N)
  instocks <- matrix(FALSE, nrow=nobs, ncol=N)
  start <- rep(0,N) # vector of starting index of each active strategy
  price <- cumprod(1+capret) # construct the price index
  for(i in 1:N) {
    signal <- MAE(price, param[1,i], param[2,i], FUN) # compute the trading signal
    signal <- signal > 0 # convert to boolean
    start[i] <- param[1,i] # current starting index
    actret[,i] <- sim_strategy(totret, rfret, signal, tc, shorts, start[i])
    instocks[,i] <- signal
  }

  # return a list with results
  result <- list(dates=dates, totret=totret, rfret=rfret, tc=tc, shorts=shorts,
                 start=start, actret=actret, desc=desc, instocks=instocks, resname="MAE")
  return(result)
}


#' Simulates the returns to the Momentum (MOM) strategy
#'
#' This function simulates the returns to the MOM strategy
#'
#' @param totret the vector of total returns
#' @param rfret the vector of risk-free returns
#' @param dates the vector of dates
#' @param capret the vector of capital gain returns
#' @param tc the amount of one-way transaction costs (a scalar)
#' @param shorts a logical value indicating the type of the strategy when a Sell signal is generated.
#' If \code{shorts=FALSE}, money is just invested in cash; otherwise, the asset is sold short
#' @param winsize a vector or a scalar that determines the size of the window used to compute the MOM
#' @return An object to be used by other functions. This object contains the returns to
#' the simulated strategies and the description of each strategy
#' @note The trading signal is computed using the vector of capital gain returns \code{capret}. If \code{capret=NULL},
#' then the trading signal is computed using the vector of total returns \code{totret}.
#' If \code{winsize} is a vector, then the function simulates a number
#' of trading strategies for all window sizes such that \code{winsize} \eqn{>1}
#' @usage sim.mom.strategy(totret, rfret, dates, capret=NULL, tc=0,
#'                         shorts = FALSE, winsize=200)
#' @family strategy simulation functions
#' @examples
#' \dontrun{
#' data <- djia.monthly # use the monthly data for DJIA index
#' dates <- index(data)
#' data <- coredata(data)
#' capret <- data[,"CAP"]
#' totret <- data[,"TOT"]
#' rfret <- data[,"RF"]
#' tc <- 0.0025 # this defines 0.25% one-way transaction costs
#' winsize <- 2:15
#' shorts <- FALSE
#' results <- sim.mom.strategy(totret=totret, rfret=rfret, dates=dates,
#'                             capret=capret, tc=tc, shorts=shorts, winsize=winsize)
#' }
#' @export sim.mom.strategy

sim.mom.strategy <- function(totret, rfret, dates, capret=NULL,
                             tc=0, shorts = FALSE, winsize=200) {

  # This function simulate the returns to
  # the MOM strategy for different sizes of the rolling window
  # the windows of size=1 are removed from the vector winsize
  # Inputs:
  # totret - vector of total returns to the passive strategy
  # rfret  - vector of returns to the risk-free asset
  # capret - vector of capital returns to the passive strategy
  # dates  - vector of dates
  # tc     - amount of transaction costs
  # shorts - defines whether to sell short (if TRUE) or stay in cash
  # winsize - vector of sizes of the rolling window
  # Remarks:
  # The trading singal is computed using "capret". If capret is missing,
  # then the trading signal is computed using "totret"
  # Output:
  # A list of objects

  # initial check for robustness
  if(!is.logical(shorts))
    stop("Argument 'shorts' must be of logical type!")
  nobs <- length(totret)
  if(length(rfret)!=nobs)
    stop("Mismatch in the number of observations between totret and rfret!")
  if(length(dates)!=nobs)
    stop("Mismatch in the number of observations between totret and dates!")
  if(length(capret)==0)
    capret <- totret
  else if(length(capret)!=nobs)
    stop("Mismatch in the number of observations between totret and capret!")

  # remove elements with winsize < 1
  winsize <- winsize[which(winsize > 1)]

  # construct the set of parameters
  N <- length(winsize)
  if(N==0) stop("Zero length of winsize vector!")
  param <- matrix(NA,nrow=3,ncol=N)
  for(i in 1:N) param[1,i] <- winsize[i]

  # construct the strategy description
  fun.name <- rep("",N)
  rule.name <- rep("MOM",N)
  desc <- data.frame(Trading.Rule=rule.name, MA.Type=fun.name,
                     Param.1 = param[1,], Param.2 = param[2,], Param.3 = param[3,])

  # construct the matrix of active returns
  actret <- matrix(nrow=nobs, ncol=N)
  instocks <- matrix(FALSE, nrow=nobs, ncol=N)
  start <- rep(0,N) # vector of starting index of each active strategy
  price <- cumprod(1+capret) # construct the price index
  for(i in 1:N) {
    signal <- MOM(price, param[1,i]) # compute the trading signal
    signal <- signal > 0 # convert to boolean
    start[i] <- param[1,i] # current starting index
    actret[,i] <- sim_strategy(totret, rfret, signal, tc, shorts, start[i])
    instocks[,i] <- signal
  }

  # return a list with results
  result <- list(dates=dates, totret=totret, rfret=rfret, tc=tc, shorts=shorts,
                 start=start, actret=actret, desc=desc, instocks=instocks, resname="MOM")
  return(result)
}

#' Simulates the returns to the Moving Average Change of Direction (CDIR) strategy
#'
#' This function simulates the returns to the CDIR strategy
#'
#' @param totret the vector of total returns
#' @param rfret the vector of risk-free returns
#' @param dates the vector of dates
#' @param capret the vector of capital gain returns
#' @param tc the amount of one-way transaction costs (a scalar)
#' @param shorts a logical value indicating the type of the strategy when a Sell signal is generated.
#' If \code{shorts=FALSE}, money is just invested in cash; otherwise, the asset is sold short
#' @param winsize a vector or a scalar that determines the size of the window used to compute a moving average
#' @param FUN a function that determines the type of a moving average
#' @return An object to be used by other functions. This object contains the returns to
#' the simulated strategies and the description of each strategy (the type of a moving average and
#' the sizes of the windows)
#' @note The function \code{FUN} is either \code{\link{SMA}}, \code{\link{LMA}}, \code{\link{EMA}},
#' \code{\link{DMA}}, \code{\link{HMA}} or \code{\link{ZMA}}. The trading signal is computed using the vector of capital gain returns \code{capret}. If \code{capret=NULL},
#' then the trading signal is computed using the vector of total returns \code{totret}.
#' If \code{winsize} is a vector, then the function simulates a number
#' of trading strategies for all window sizes such that \code{winsize} \eqn{>1}
#' @usage sim.cdir.strategy(totret, rfret, dates, capret=NULL, tc=0,
#'                          shorts = FALSE, winsize=200, FUN=SMA)
#' @family strategy simulation functions
#' @examples
#' \dontrun{
#' data <- djia.monthly # use the monthly data for DJIA index
#' dates <- index(data)
#' data <- coredata(data)
#' capret <- data[,"CAP"]
#' totret <- data[,"TOT"]
#' rfret <- data[,"RF"]
#' tc <- 0.0025 # this defines 0.25% one-way transaction costs
#' winsize <- 2:15
#' shorts <- FALSE
#' results <- sim.cdir.strategy(totret=totret, rfret=rfret, dates=dates, capret=capret,
#'                              tc=tc, shorts=shorts, winsize=winsize, FUN=LMA)
#' }
#' @export sim.cdir.strategy

sim.cdir.strategy <- function(totret, rfret, dates, capret=NULL,
                             tc=0, shorts = FALSE, winsize=200, FUN=SMA) {

  # This function simulate the returns to
  # the CDIR strategy for different sizes of the rolling window
  # the windows of size=1 are removed from the vector winsize
  # Inputs:
  # totret - vector of total returns to the passive strategy
  # rfret  - vector of returns to the risk-free asset
  # capret - vector of capital returns to the passive strategy
  # dates  - vector of dates
  # tc     - amount of transaction costs
  # shorts - defines whether to sell short (if TRUE) or stay in cash
  # winsize - vector of sizes of the rolling window
  # Remarks:
  # The trading singal is computed using "capret". If capret is missing,
  # then the trading signal is computed using "totret"
  # Output:
  # A list of objects

  # initial check for robustness
  if(!is.logical(shorts))
    stop("Argument 'shorts' must be of logical type!")
  nobs <- length(totret)
  if(length(rfret)!=nobs)
    stop("Mismatch in the number of observations between totret and rfret!")
  if(length(dates)!=nobs)
    stop("Mismatch in the number of observations between totret and dates!")
  if(length(capret)==0)
    capret <- totret
  else if(length(capret)!=nobs)
    stop("Mismatch in the number of observations between totret and capret!")
  if(!is.numeric(tc))
    stop("Something is wrong. Argument 'tc' is not of a numerical type!")
  if(!is.numeric(winsize))
    stop("Something is wrong. Argument 'winsize' is not of a numerical type!")

  # remove elements with winsize < 1
  winsize <- winsize[which(winsize > 1)]

  # construct the set of parameters
  N <- length(winsize)
  if(N==0) stop("Zero length of winsize vector!")
  param <- matrix(NA,nrow=3,ncol=N)
  for(i in 1:N) param[1,i] <- winsize[i]

  # construct the strategy description
  fun.name <- rep(deparse(substitute(FUN)),N)
  rule.name <- rep("CDIR",N)
  desc <- data.frame(Trading.Rule=rule.name, MA.Type=fun.name,
                     Param.1 = param[1,], Param.2 = param[2,], Param.3 = param[3,])

  # construct the matrix of active returns
  actret <- matrix(nrow=nobs, ncol=N)
  instocks <- matrix(FALSE, nrow=nobs, ncol=N)
  start <- rep(0,N) # vector of starting index of each active strategy
  price <- cumprod(1+capret) # construct the price index
  for(i in 1:N) {
    ma <- FUN(price, param[1,i]) # compute the moving average
    signal <- c(0,diff(ma)) # compute the trading signal
    signal <- signal > 0 # convert to boolean
    start[i] <- param[1,i] # current starting index
    actret[,i] <- sim_strategy(totret, rfret, signal, tc, shorts, start[i])
    instocks[,i] <- signal
  }

  # return a list with results
  result <- list(dates=dates, totret=totret, rfret=rfret, tc=tc, shorts=shorts,
                 start=start, actret=actret, desc=desc, instocks=instocks, resname="CDIR")
  return(result)
}

#' Simulates the returns to the Moving Average Convergence Divergence (MACD) strategy
#'
#' This function simulates the returns to the MACD strategy
#'
#' @param totret the vector of total returns
#' @param rfret the vector of risk-free returns
#' @param dates the vector of dates
#' @param capret the vector of capital gain returns
#' @param tc the amount of one-way transaction costs (a scalar)
#' @param shorts a logical value indicating the type of the strategy when a Sell signal is generated.
#' If \code{shorts=FALSE}, money is just invested in cash; otherwise, the asset is sold short
#' @param fast a vector or a scalar that determines the size of the shorter averaging window
#' @param slow a vector or a scalar that determines the size of the longer averaging window
#' @param final a vector or a scalar that determines the size of the window for final smoothing
#' @param FUN a function that determines the type of a moving average
#' @return An object to be used by other functions. This object contains the returns to
#' the simulated strategies and the description of each strategy (the type of a moving average,
#' the sizes of the shorter and the longer windows, the size of the window for final smoothing)
#' @note The function \code{FUN} is either \code{\link{SMA}}, \code{\link{LMA}}, \code{\link{EMA}},
#' \code{\link{DMA}}, \code{\link{HMA}} or \code{\link{ZMA}}. The trading signal is computed using the vector of capital gain returns \code{capret}. If \code{capret=NULL},
#' then the trading signal is computed using the vector of total returns \code{totret}.
#' If one of the arguments \code{fast}, \code{slow} or \code{final} is a vector, then the function simulates a number
#' of trading strategies for all combinations of \code{fast}, \code{slow} and \code{final}
#'  such that \code{fast} \eqn{<} \code{slow} and \code{final} \eqn{>1}.
#' @usage sim.macd.strategy(totret, rfret, dates, capret=NULL, tc=0, shorts = FALSE,
#'                          fast=12, slow=26, final=9, FUN=EMA)
#' @family strategy simulation functions
#' @examples
#' \dontrun{
#' data <- djia.daily # use the daily data for DJIA index
#' dates <- index(data)
#' data <- coredata(data)
#' capret <- data[,"CAP"]
#' totret <- data[,"TOT"]
#' rfret <- data[,"RF"]
#' tc <- 0.0025 # this defines 0.25% one-way transaction costs
#' fast <- 1:15 # define the sizes of the fast (shorter) averaging window
#' slow <- 2:45 # define the sizes of the slow (longer) averaging window
#' final <- 2:20 # defines the sizes of the window for final smoothing
#' shorts <- FALSE
#' results <- sim.macd.strategy(totret=totret, rfret=rfret, dates=dates,
#'  capret=capret, tc=tc, shorts=shorts, fast=fast, slow=slow,
#'  final=final, FUN=EMA)
#' }
#' @export sim.macd.strategy

sim.macd.strategy <- function(totret, rfret, dates, capret=NULL,
                             tc=0, shorts = FALSE, fast=12, slow=26, final=9, FUN=EMA) {

  # This function simulate the returns to
  # the MAC strategy for different sizes of fast and slow moving averages
  # Inputs:
  # totret - vector of total returns to the passive strategy
  # rfret  - vector of returns to the risk-free asset
  # capret - vector of capital returns to the passive strategy
  # dates  - vector of dates
  # tc     - amount of transaction costs
  # shorts - defines whether to sell short (if TRUE) or stay in cash
  # fast   - vector of sizes of the fast averaging window
  # slow   - vector of sizes of the slow averaging window
  # final - vector of sizes of the averaging window for final smoothing
  # FUN    - function that computes the moving average
  # Remarks:
  # The trading singal is computed using "capret". If capret is missing,
  # then the trading signal is computed using "totret"
  # Function simulates the valid strategies where fast < slow
  # Output:
  # A list of objects

  # initial check for robustness
  if(!is.logical(shorts))
    stop("Argument 'shorts' must be of logical type!")
  nobs <- length(totret)
  if(length(rfret)!=nobs)
    stop("Mismatch in the number of observations between totret and rfret!")
  if(length(dates)!=nobs)
    stop("Mismatch in the number of observations between totret and dates!")
  if(length(capret)==0)
    capret <- totret
  else if(length(capret)!=nobs)
    stop("Mismatch in the number of observations between totret and capret!")
  fun.name <- deparse(substitute(FUN))
  if(!exists(fun.name, mode='function')) {
    err.str <- paste("Something is wrong. Function", fun.name, "cannot be found!")
    stop(err.str)
  }

  # remove elements with values < 1
  slow <- slow[which(slow > 1)]
  # remove elements with values < 0
  fast <- fast[which(fast > 0)]
  # remove elements with values < 1
  final <- final[which(final > 1)]

  # compute the lengths
  n.slow <- length(slow)
  n.fast <- length(fast)
  n.final <- length(final)
  if(n.slow==0) stop("Zero length of slow vector!")
  if(n.fast==0) stop("Zero length of fast vector!")
  if(n.final==0) stop("Zero length of final vector!")

  # construct the initial set of parameters
  N <- n.slow*n.fast*n.final
  param <- matrix(NA,nrow=3,ncol=N)
  for(i in 1:n.fast)
    for(j in 1:n.slow)
      for(k in 1:n.final){
      ind <- (i-1)*n.slow*n.final + (j-1)*n.final + k
      param[1,ind] <- fast[i]
      param[2,ind] <- slow[j]
      param[3,ind] <- final[k]
      }

  # delete the pairs of parameters where fast >= slow
  ind <- which(param[2,] > param[1,])
  if(length(ind)==1) param <- as.matrix(param[,ind],nrow=3,ncol=1) else param <- param[,ind]

  # construct the strategy description
  N <- ncol(param)
  fun.name <- rep(deparse(substitute(FUN)),N)
  rule.name <- rep("MACD",N)
  desc <- data.frame(Trading.Rule=rule.name, MA.Type=fun.name,
                     Param.1 = param[1,], Param.2 = param[2,], Param.3 = param[3,])

  # construct the matrix of active returns
  actret <- matrix(nrow=nobs, ncol=N)
  instocks <- matrix(FALSE, nrow=nobs, ncol=N)
  start <- rep(0,N) # vector of starting index of each active strategy
  price <- cumprod(1+capret) # construct the price index
  for(i in 1:N) {
    mac <- FUN(price, param[1,i]) - FUN(price, param[2,i]) # compute the mac signal
    signal <- mac - FUN(mac, param[3,i]) # compute the trading signal
    signal <- signal > 0 # convert to boolean
    start[i] <- param[2,i] + param[3,i]  # current starting index
    actret[,i] <- sim_strategy(totret, rfret, signal, tc, shorts, start[i])
    instocks[,i] <- signal
  }

  # return a list with results
  result <- list(dates=dates, totret=totret, rfret=rfret, tc=tc, shorts=shorts,
                 start=start, actret=actret, desc=desc, instocks=instocks, resname="MACD")
  return(result)
}

is.desc <- function(x) {
  # check whether x is the result of a strategy simulation
  if(!exists("Trading.Rule", where=x)) {
    cat("Element 'dates' absent from the simulation results!")
    return(FALSE)
  }
  if(!exists("MA.Type", where=x)) {
    cat("Element 'MA.Type' absent from the simulation results!")
    return(FALSE)
  }
  if(!exists("Param.1", where=x)) {
    cat("Element 'Param.1' absent from the simulation results!")
    return(FALSE)
  }
  if(!exists("Param.2", where=x)) {
    cat("Element 'Param.2' absent from the simulation results!")
    return(FALSE)
  }
  if(!exists("Param.3", where=x)) {
    cat("Element 'Param.3' absent from the simulation results!")
    return(FALSE)
  }
  return(TRUE)
}

is.sim.results <- function(x) {
  # check whether x is the result of a strategy simulation
  if(!exists("dates", where=x)) {
    cat("Element 'dates' absent from the description!")
    return(FALSE)
  }
  if(!exists("totret", where=x)) {
    cat("Element 'totret' absent from the simulation results!")
    return(FALSE)
  }
  if(!exists("rfret", where=x)) {
    cat("Element 'rfret' absent from the simulation results!")
    return(FALSE)
  }
  if(!exists("tc", where=x)) {
    cat("Element 'tc' absent from the simulation results!")
    return(FALSE)
  }
  if(!exists("shorts", where=x)) {
    cat("Element 'shorts' absent from the simulation results!")
    return(FALSE)
  }
  if(!exists("start", where=x)) {
    cat("Element 'start' absent from the simulation results!")
    return(FALSE)
  }
  if(!exists("actret", where=x)) {
    cat("Element 'actret' absent from the simulation results!")
    return(FALSE)
  }
  if(!exists("desc", where=x)) {
    cat("Element 'desc' absent from the simulation results!")
    return(FALSE)
  }
  # if(!exists("instocks", where=x)) {
  #   cat("Element 'instocks' absent from the simulation results!")
  #   return(FALSE)
  # }
  return(TRUE)
}

#' Merges the results of simulating the returns to different trading strategies
#'
#' This function merges the results of simulating the returns to different trading strategies
#'
#' @param x the result of simulation of a trading strategy. Specifically, this is the
#' result returned by function \code{\link{sim.mom.strategy}},
#' \code{\link{sim.mac.strategy}}, \code{\link{sim.cdir.strategy}}, or \code{\link{sim.macd.strategy}}
#' @param y the result of simulation of another trading strategy. Again, this is the
#' result returned by function \code{\link{sim.mom.strategy}},
#' \code{\link{sim.mac.strategy}}, \code{\link{sim.cdir.strategy}}, or \code{\link{sim.macd.strategy}}
#' @param ... other possible results of simulations if more than two results must be merged
#' @return An object to be used by other functions. This object is a result of merging two or
#' multiple results of simulating the returns to different trading strategies
#' @usage sim.results.merge(x, y, ...)
#' @examples
#' # Suppose that "res1", "res2", and "res3" are the results of simulations of
#' # three different trading strategies. The following command merges the three results
#' \dontrun{
#' results <- sim.results.merge(res1, res2, res3)
#' }
#' @export sim.results.merge

sim.results.merge <- function(x, y, ...) {
  # this function merges several simulated strategies into one
  # Inputs
  # x and y are the resuts of simulations
  # Outpue
  # A combined result of simulations

  if(!is.sim.results(x))
    stop("Something is wrong, argument 'x' is not a result of strategy simulation!")
  if(!is.sim.results(y))
    stop("Something is wrong, argument 'y' is not a result of strategy simulation!")

  # merge x and y
  dates <- x$dates
  if(!all(dates %in% y$dates))
    stop("Different vectors of dates in x and y!")
  totret <- x$totret
  rfret <- x$rfret
  tc <- x$tc
  shorts <- x$shorts

  # merge descriptions
  desc <- x$desc
  if(!all(colnames(desc) %in% colnames(y$desc)))
    stop("Different column names in desc!")
  desc <- rbind(desc, y$desc)
  # merge active returns
  actret <- x$actret
  actret <- cbind(actret, y$actret)
  # merge "instocks" data
  instocks <- x$instocks
  instocks <- cbind(instocks, y$instocks)
  # merge start indices
  start <- x$start
  start <- c(start, y$start)


  # see if there are other strategies to merge
  arguments <- list(...)
  n <- length(arguments)
  if(n > 0) {
    for(i in 1:n) {
      z <- arguments[[i]]
      if(!is.sim.results(z))
        stop("Something is wrong, argument 'z' is not a result of strategy simulation!")
      if(!all(dates %in% z$dates))
        stop("Different vectors of dates!")
      # merge descriptions
      if(!all(colnames(desc) %in% colnames(z$desc)))
        stop("Different column names!")
      desc <- rbind(desc, z$desc)
      # merge active returns
      actret <- cbind(actret, z$actret)
      # merge "instocks" data
      instocks <- cbind(instocks, z$instocks)
      # merge start indices
      start <- c(start, z$start)
    }
  }
  # return a list with results
  result <- list(dates=dates, totret=totret, rfret=rfret, tc=tc, shorts=shorts,
                 start=start, actret=actret, desc=desc, instocks=instocks)
  return(result)
}

#' Finds the optimal trading strategies in a back-test (in-sample test)
#'
#' This function finds the optimal trading strategies in a back-test
#'
#' @param results the results of simulation of a trading strategy or multiple trading strategies.
#' Specifically, this is the result returned by function \code{\link{sim.mom.strategy}},
#' \code{\link{sim.mac.strategy}}, \code{\link{sim.cdir.strategy}}, \code{\link{sim.macd.strategy}},
#'  or \code{\link{sim.results.merge}}
#' @param start.date the start of the in-sample period
#' @param end.date the end of the in-sample period
#' @param n.first the (maximum) number of the best performing strategies in the returned object
#' @param include.bh a logical value indicating whether to consider the Buy-and-Hold (bh) strategy
#' in addition to the moving average strategies
#' @param FUN the function that computes the performance of a trading strategy.
#' @return An object to be used by other functions
#' @note The argument \code{results} contains the dates vector that defines the historical period
#' over which the trading strategies are simulated. The function \code{FUN} is either \code{\link{Excret}},
#' \code{\link{Sharpe}}, or \code{\link{Sortino}}.
#' @usage back.test(results, start.date, end.date, n.first=7, include.bh=FALSE, FUN=Sharpe)
#' @family strategy testing functions
#' @examples
#' \dontrun{
#' library(matiming)
#' library(xtable)
#' library(zoo)
#' # access the data
#' data <- sp500.monthly
#' dates <- index(data)
#' data <- coredata(data)
#' capret <- data[,"CAP"]
#' totret <- data[,"TOT"]
#' rfret <- data[,"RF"]
#' # simulate the trading strategies
#' tc <- 0.0025
#' fast <- 1:5
#' slow <- 2:15
#' shorts <- F
#' results <- sim.mac.strategy(totret=totret, rfret=rfret, dates=dates,
#'                          capret=capret, tc=tc, shorts=shorts, fast=fast, slow=slow, FUN=SMA)
#' # define the historical period
#' start.date <- as.Date("1950-01-01", format="%Y-%m-%d")
#' end.date <- as.Date("2015-12-31", format="%Y-%m-%d")
#' n.first <- 5
#' include.bh <- TRUE
#' res <- back.test(results, start.date=start.date, end.date=end.date,
#'                        n.first=n.first, include.bh=include.bh, FUN=Sortino)
#' # print out the ranking of trading strategies
#' df <- best.is.strategies(res, to.annual=sqrt(12), plot.results=TRUE)
#' xtab <- xtable(df, digits=3)
#' print(xtab, include.rownames=FALSE)
#' }
#' @export back.test

back.test <- function(results, start.date, end.date, n.first=7, include.bh=FALSE, FUN=Sharpe) {
  # this function finds the "in-sample" optimal strategy
  # optimization criterion is given by fanction FUN
  # Inputs:
  # results - a list of simulation results
  # start.date  - the starting date of a historical period
  # end.date    - the ending date of a historical period
  # n.first - the number of best performing strategies
  # FUN    - the optimality criterion

  if(!is.sim.results(results))
    stop("Something is wrong. Argument 'results' is not a result of strategy simulation!")

  if(!is.numeric(n.first))
    stop("Something is wrong. Argument 'n.first' is not of a numerical type!")

  if(!is.logical(include.bh))
    stop("Argument 'include.bh' must be of logical type!")

  fun.name <- deparse(substitute(FUN))
  if(!exists(fun.name, mode='function')) {
    err.str <- paste("Something is wrong. Function", fun.name, "cannot be found!")
    stop(err.str)
  }

  # access the elements of the list
  desc <- results$desc
  actret <- results$actret
  start <- results$start
  dates <- results$dates
  totret <- results$totret
  rfret <- results$rfret
  tc <- results$tc
  shorts <- results$shorts

  if(include.bh) {
    # add the buy-and-hold strategy to the list of strategies
    actret <- cbind(actret, totret) # add the BH strategy to the matrix of returns
    # add the description for BH strategy
    new.row <- data.frame(Trading.Rule="B&H", MA.Type="",
                          Param.1 = NA, Param.2 = NA, Param.3 = NA)
    desc <- rbind(desc, new.row)
  }

  # select the indices of the required observations
  date.ind <- dates >= start.date & dates <= end.date

  if(is.na(which(date.ind)[1]))
    stop("Something is wrong. The requested period has zero length!")

  # check that the first date index is greater than the max start index
  max.start <- max(start)
  start.date.ind <- which(date.ind)[1]
  if(start.date.ind < max.start)
    stop("The start.date is too early!")

  # select the required period
  actret <- actret[date.ind,]
  totret <- totret[date.ind]
  rfret <- rfret[date.ind]
  dates <- dates[date.ind]

  # check that the requested period has non-zero length
  islen <- length(totret)
  if(islen==0)
    stop("Something is wrong. The vector of returns has zero length!")

  # compute the performance measures
  if(is.matrix(actret)) {
    n <- ncol(actret) # this is the number of simulated strategies
    PM <- rep(0,n)
    for(i in 1:n) PM[i] <- FUN(actret[,i]-rfret)
    # sort the strategies with respect to performance
    strat.ranks <- sort.int(PM, decreasing = TRUE, index.return = TRUE,
                            method = "radix", na.last = TRUE)$ix
    n.first <- min(n, n.first) # select the minimum
  } else {
    n.first <- 1
    PM <- FUN(actret-rfret)
    strat.ranks <- 1
    actret <- as.matrix(actret,nrow=length(actret),ncol=1)
  }

  # construct the returned objects
  top.PM <- rep(0,n.first)
  top.actret <- matrix(nrow=islen, ncol=n.first)
  rule.name <- rep("",n.first)
  ma.name <- rep("",n.first)
  param <- matrix(NA,nrow=3,ncol=n.first)
  for(i in 1:n.first) {
    top.PM[i] <- PM[strat.ranks[i]]
    top.actret[,i] <- actret[,strat.ranks[i]]
    rule.name[i] <- as.character(desc$Trading.Rule[strat.ranks[i]])
    ma.name[i] <- as.character(desc$MA.Type[strat.ranks[i]])
    param[1,i] <- desc$Param.1[strat.ranks[i]]
    param[2,i] <- desc$Param.2[strat.ranks[i]]
    param[3,i] <- desc$Param.3[strat.ranks[i]]
  }

  desc <- data.frame(Trading.Rule=rule.name, MA.Type=ma.name,
                     Param.1 = param[1,], Param.2 = param[2,], Param.3 = param[3,],
                     PM = top.PM)

  # return a list with results
  result <- list(dates=dates, totret=totret, rfret=rfret, FUN=FUN, fun.name=fun.name,
                 start=start, actret=top.actret, desc=desc, pm.bh=FUN(totret-rfret),
                 start.date=start.date, end.date=end.date, tc=tc, shorts=shorts, resname="IS")
  return(result)
}

outstr <- function(x, digits=2) {
  # prints out the value
  str <- paste(formatC(x, digits=digits, format = "f", flag = "0"))
}

make.strat.names <- function(desc) {
  # create the word names for each strategy
  n <- nrow(desc)
  for(i in 1:n) {
    # access the elements of the data frame
    rule <- as.character(desc$Trading.Rule[i])
    ma <- as.character(desc$MA.Type[i])
    par.1 <- desc$Param.1[i]
    par.2 <- desc$Param.2[i]
    par.3 <- desc$Param.3[i]
    # construct the string
    if(rule == "MOM") {
      str <- paste("MOM(")
    } else {
      if(rule == "B&H") {
        str <- paste("B&H(")
      } else str <- paste(substr(ma,1,1),rule,"(",sep="")
    }
    if(!is.na(par.1)) str <- paste(str, par.1, sep="")
    if(!is.na(par.2)) str <- paste(str,",",par.2, sep="")
    if(!is.na(par.3)) str <- paste(str,",",par.3, sep="")
    str <- paste(str,")", sep="")
    if( (rule=="MAC") & (par.1==1) ) {
      # this is P-MA rule
      str <- paste("P-",substr(ma,1,1),"MA(",par.2,")", sep="")
    }
    if(i==1) text <- str else text <- c(text, str)
  }
  return(text)
}

#' Reports the best trading strategies in a back-test (in-sample test)
#'
#' This function reports the best trading strategies in a back-test
#'
#' @param results the results returned by function \code{\link{back.test}}
#' @param to.annual a coefficient used to annualize the performance measure. For example, when
#' the performance is measured using either \code{\link{Sharpe}} or \code{\link{Sortino}},
#' use \code{sqrt(252)} and \code{sqrt(12)} when the returns are at the daily
#' and monthly frequency respectively. If the performance is measred by \code{\link{Excret}}, use \code{12}
#' @param plot.results a logical value indicating whether to plot the cumulative returns to
#' the best trading strategies and to the buy-and-hold strategy
#' @return An object of class \code{data.frame} containing three columns: \code{Rank}, \code{Rule}, and \code{Performance}.
#' @usage best.is.strategies(results, to.annual=sqrt(12), plot.results=FALSE)
#' @family strategy summary functions
#' @examples
#' \dontrun{
#' library(matiming)
#' library(xtable)
#' library(zoo)
#'
#' data <- sp500.monthly
#' dates <- index(data)
#' data <- coredata(data)
#' capret <- data[,"CAP"]
#' totret <- data[,"TOT"]
#' rfret <- data[,"RF"]
#'
#' tc <- 0.0025
#' fast <- 1:9
#' slow <- 2:18
#'
#' res.mac <- sim.mac.strategy(totret=totret, rfret=rfret, dates=dates,
#'                             capret=capret, tc=tc, fast=fast, slow=slow, FUN=SMA)
#'
#' res.mom <- sim.mom.strategy(totret=totret, rfret=rfret, dates=dates,
#'                             capret=capret, tc=tc, winsize=slow)
#'
#' results <- sim.results.merge(res.mac, res.mom)
#'
#' start.date <- as.Date("1950-01-01", format="%Y-%m-%d")
#' end.date <- as.Date("2010-12-31", format="%Y-%m-%d")
#' n.first <- 10
#' include.bh <- TRUE
#'
#' res <- back.test(results, start.date=start.date, end.date=end.date,
#'                        n.first=n.first, include.bh=include.bh, FUN=Sharpe)
#'
#' df <- best.is.strategies(res, plot.results=TRUE)
#' xtab <- xtable(df, digits=3)
#' print(xtab, include.rownames=FALSE)
#' }
#' @export best.is.strategies

best.is.strategies <- function(results, to.annual=sqrt(12), plot.results=FALSE) {
  # this function prints out the "in-sample" optimal strategies
  # Inputs:
  # result - the result returned by function "back.test"
  # digits - the number of digits to print out the performance measure
  # to.annual - annualization factor for the performance measure
  # plot.results - whether to produce the plot

  if(results$resname != "IS")
    stop("results object must be produced by back.test function!")

  if(!is.logical(plot.results))
    stop("Argument 'plot.results' must be of logical type!")

  if(!is.numeric(to.annual))
    stop("Something is wrong. Argument 'to.annual' is not of a numerical type!")

  # construct the cumulative returns
  desc <- results$desc
  n <- nrow(desc)
  text <- make.strat.names(desc)
  if(!("B&H()" %in% text)) {
  df <- data.frame(Rank=c(1:n,NA),
                   Rule=c(text,"B&H()"),
                   Performance=c(desc$PM*to.annual,results$pm.bh*to.annual) )
  } else {
    df <- data.frame(Rank=c(1:n),
                     Rule=text,
                     Performance=desc$PM*to.annual)
  }

  if(plot.results==TRUE) {
    actret <- results$actret
    cumret <- matrix(nrow=nrow(actret),ncol=n+1)
    cumret[,1] <- 100*cumprod(1+results$totret)
    for(i in 2:(n+1)) cumret[,i] <- 100*cumprod(1+actret[,i-1])
    data <- zoo(log(cumret), order.by=results$dates)
    plot(data, plot.type="single", col=1:(n+1), xlab="",
         ylab="Log cumulative return", lwd=1)
    legend.str <- c("B&H", text)
    legend("bottomright", legend.str, col=1:(n+1), lty=1, bty = "n",
           cex=0.9, lwd=1)
  }

  return(df)
}


#' Plots the trading strategies rolling outperformance
#'
#' This function plots the trading strategies rolling outperformance
#'
#' @param results the results of the back-test or simulation of a trading strategy or multiple trading strategies.
#' The former is the result returned by function \code{\link{back.test}}. The latter is the result returned by function \code{\link{sim.mom.strategy}},
#' \code{\link{sim.mac.strategy}}, \code{\link{sim.cdir.strategy}}, \code{\link{sim.macd.strategy}},
#'  or \code{\link{sim.results.merge}}
#' @param start.date the start of the first rolling period
#' @param split.date the end of the first rolling period
#' @param end.date the end of the total period
#' @param FUN the function that computes the performance of a trading strategy. The function \code{FUN} is either \code{\link{Excret}},
#' \code{\link{Sharpe}}, or \code{\link{Sortino}}.
#' @return None
#' @note The argument \code{results} contains the dates vector that defines the historical period
#' over which the trading strategies are simulated. The function \code{FUN} is only used if \code{results} is the result of simulation.
#' The outperformance is defined as the difference between the performance of a trading strategy and the performance of
#' the corresponding passive strategy. The number of plotted strategies is limited by 5.
#' @usage is.outperformance.plot(results, start.date, split.date, end.date, FUN=Sharpe)
#' @family plotting functions
#' @examples
#' \dontrun{
#' library(matiming)
#' library(zoo)
#' # access the data
#' data <- sp500.monthly
#' dates <- index(data)
#' data <- coredata(data)
#' capret <- data[,"CAP"]
#' totret <- data[,"TOT"]
#' rfret <- data[,"RF"]
#' # simulate the SMA(10) strategy
#' tc <- 0.0025
#' fast <- 1
#' slow <- 10
#' shorts <- F
#' results <- sim.mac.strategy(totret=totret, rfret=rfret, dates=dates,
#'                          capret=capret, tc=tc, shorts=shorts, fast=fast, slow=slow, FUN=SMA)
#' # plot the rolling outperformance
#' start.date <- as.Date("1920-01-01", format="%Y-%m-%d")
#' split.date <- as.Date("1929-12-31", format="%Y-%m-%d")
#' end.date <- as.Date("2015-12-31", format="%Y-%m-%d")
#' is.outperformance.plot(results, start.date=start.date, split.date=split.date,
#'  end.date=end.date, FUN=Sharpe)
#' }
#' @export is.outperformance.plot

is.outperformance.plot <- function(results, start.date, split.date, end.date, FUN=Sharpe) {
  # optimization criterion is given by fanction FUN
  # Inputs:
  # results - a list of simulation results
  # start.date  - the starting date of a historical period
  # end.date    - the ending date of a historical period
  # step.size -
  # FUN    - the optimality criterion

  if( !(is.sim.results(results) | (results$resname == "IS")) )
    stop("Something is wrong. Argument 'results' is not a result of strategy simulation and not the results produced by back.test function!")

  # access the elements of the list
  desc <- results$desc
  actret <- results$actret
  start <- results$start
  dates <- results$dates
  totret <- results$totret
  rfret <- results$rfret

  if(is.sim.results(results)) {
    fun.name <- deparse(substitute(FUN))
    if(!exists(fun.name, mode='function')) {
      err.str <- paste("Something is wrong. Function", fun.name, "cannot be found!")
      stop(err.str)
    }
  } else {
    fun.name <- results$fun.name
    FUN <- results$FUN
  }

  # select the indices of the required observations
  date.ind <- dates >= start.date & dates <= split.date

  if(is.na(which(date.ind)[1]))
    stop("Something is wrong. The period from 'start' to 'split' has zero length!")

  # check that the first date index is greater than the max start index
  max.start <- max(start)
  start.rolling <- which(date.ind)[1]
  if(start.rolling < max.start)
    stop("The start.date is too early!")

  dates.out <- dates > split.date & dates <= end.date
  ind.dates.out <- which(dates.out) # indices of the OOS period
  start.out <- ind.dates.out[1]
  if(is.na(start.out))
    stop("Something is wrong. The period from 'split' to 'end' has zero length!")
  nout <- length(ind.dates.out) # the length of the OUT period
  end.out <- ind.dates.out[nout]
  end.rolling <- start.out - 1

  N <- nout # the number of rolling windows
  n <- ncol(actret)
  max.cols <- 5
  if(n > max.cols) n <- max.cols

  snames <- make.strat.names(desc)

  # allocate variables
  dPM <- matrix(nrow=N, ncol=n) # to hold the difference in performance measures
  actret <- actret - rfret
  totret <- totret - rfret

  # main loop to compute the rolling outperformance
  for(i in 1:N) {
    # compute the performance measures in the rolling IS period
    for(j in 1:n) dPM[i,j] <- FUN(actret[start.rolling:end.rolling,j]) - FUN(totret[start.rolling:end.rolling])
    # move the rolling window
    start.rolling <- start.rolling + 1
    end.rolling <- end.rolling + 1
  }

  n.years <- round(as.numeric(difftime(split.date, start.date, units="weeks"))/52.25)

  dates <- dates[dates > split.date & dates <= end.date]
  data <- zoo(dPM, order.by=dates)

  plot(data, plot.type="single", col=2:(n+1), lwd=1, ylab="Outperformance",
       main=paste("Rolling ",n.years," - year outperformance",sep=""),
       xlab="")
  lines(dates, rep(0,N), lty=2)
  legend("top", legend=snames[1:min(n,length(snames))], col=2:(n+1), lty=1, lwd=1, bty = "n",
         cex=0.9, ncol=n)
}

#' Simulates the returns to the out-of-sample trading strategy
#'
#' This function simulates the returns to the out-of-sample trading strategy
#'
#' @param results the results of simulation of a trading strategy or multiple trading strategies.
#' Specifically, this is the result returned by function \code{\link{sim.mom.strategy}},
#' \code{\link{sim.mac.strategy}}, \code{\link{sim.cdir.strategy}}, \code{\link{sim.macd.strategy}},
#'  or \code{\link{sim.results.merge}}
#' @param start.date the start of the in-sample period
#' @param end.is.date the end of the initial in-sample period
#' @param end.oos.date the end of the out-of-sample period
#' @param refit.every this parameter determines how often the best trading strategy in a back test is determined
#' @param rolling a logical value indicating whether to use the expanding (if FALSE)
#'  or rolling (if TRUE) in-sample window
#' @param include.bh a logical value indicating whether to consider the Buy-and-Hold (bh) strategy
#' in addition to the moving average strategies
#' @param FUN the function that computes the performance of a trading strategy
#' @return An object to be used by other functions
#' @note The argument \code{results} contains the dates vector that defines the historical period
#' over which the trading strategies are simulated. The function \code{FUN} is either \code{\link{Excret}},
#' \code{\link{Sharpe}}, or \code{\link{Sortino}}.
#' This function only simulates the out-of-sample strategy; to test the outperformance
#' hypothesis, use either function \code{\link{outperformance.test}} or function \code{\link{descriptive.statistics}.}
#' @usage forward.test(results, start.date, end.is.date, end.oos.date,
#'                     refit.every=1, rolling=FALSE, include.bh=FALSE, FUN=Sharpe)
#' @family strategy testing functions
#' @examples
#' \dontrun{
#' library(matiming)
#' library(xtable)
#' library(zoo)
#'
#' # access the data
#' data <- sp500.monthly
#' dates <- index(data)
#' data <- coredata(data)
#' capret <- data[,"CAP"]
#' totret <- data[,"TOT"]
#' rfret <- data[,"RF"]
#'
#' # simulate the trading strategies
#' tc <- 0.0025
#' fast <- 1:5
#' slow <- 2:15
#' shorts <- F
#' results <- sim.mac.strategy(totret=totret, rfret=rfret, dates=dates,
#'                          capret=capret, tc=tc, shorts=shorts, fast=fast, slow=slow, FUN=SMA)
#'
#' # perform the out-of-sample test
#' start.date <- as.Date("1930-01-01", format="%Y-%m-%d")
#' end.is.date <- as.Date("1949-12-31", format="%Y-%m-%d")
#' end.oos.date <- as.Date("2015-12-31", format="%Y-%m-%d")
#' refit.every <- 5
#' include.bh <- TRUE
#' rolling <- TRUE
#' res <- forward.test(results, start.date=start.date, end.is.date=end.is.date,
#'  end.oos.date=end.oos.date, refit.every=refit.every, rolling=rolling,
#'  include.bh=include.bh, FUN=Sortino)
#'
#' # report the descriptive statistics and the p-value of the test
#' df <- descriptive.statistics(res)
#' xtab <- xtable(df, digits=2)
#' print(xtab)
#' }
#' @export forward.test

forward.test <- function(results, start.date, end.is.date, end.oos.date,
                         refit.every=1, rolling=FALSE, include.bh=FALSE, FUN=Sharpe) {
  # this function simulates the returns to the "out-of-sample" strategy
  # optimization criterion is given by fanction FUN
  # Inputs:
  # results       - a list of simulation results for different trading rules
  # start.date    - the starting date of a historical period; start of in-sample period
  # end.is.date   - the ending date of the initial in-sample period
  # end.oos.date  - the ending date of the out-of-sample period
  # refit.every   - determines how often the optimal strategy is refitted (the number of steps)
  # rolling       - defines whether to use rolling or expanding estimation scheme
  # FUN    - the optimality criterion

  if(!is.sim.results(results))
    stop("Something is wrong. Argument 'results' is not a result of strategy simulation!")

  fun.name <- deparse(substitute(FUN))
  if(!exists(fun.name, mode='function')) {
    err.str <- paste("Something is wrong. Function", fun.name, "cannot be found!")
    stop(err.str)
  }

  if(!is.logical(rolling))
    stop("Argument 'rolling' must be of logical type!")

  if(!is.logical(include.bh))
    stop("Argument 'include.bh' must be of logical type!")

  if(!is.numeric(refit.every))
    stop("Something is wrong. Argument 'refit.every' is not of a numerical type!")

  # access the elements of the list
  desc <- results$desc
  actret <- results$actret
  start <- results$start
  dates <- results$dates
  totret <- results$totret
  rfret <- results$rfret
  instocks <- results$instocks
  tc <- results$tc
  shorts <- results$shorts

  if(include.bh) {
    # add the buy-and-hold strategy to the list of strategies
    actret <- cbind(actret, totret) # add the BH strategy to the matrix of returns
    # add the description for BH strategy
    new.row <- data.frame(Trading.Rule="B&H", MA.Type="",
                          Param.1 = NA, Param.2 = NA, Param.3 = NA)
    desc <- rbind(desc, new.row)
    # add a column with TRUE to 'instocks'
    instocks <- cbind(instocks, rep(TRUE,length(totret)))
  }

  if(!is.matrix(actret))
    stop("There is only one simulated strategy!")

  # find the initial indices of each period
  dates.is <- dates >= start.date & dates <= end.is.date
  dates.oos <- dates > end.is.date & dates <= end.oos.date
  start.is <- which(dates.is)[1]
  if(is.na(start.is))
    stop("Something is wrong. The in-sample period has zero length!")
  ind.dates.oos <- which(dates.oos) # indices of the OOS period
  start.oos <- ind.dates.oos[1]
  if(is.na(start.oos))
    stop("Something is wrong. The out-of-sample period has zero length!")
  noos <- length(ind.dates.oos) # the length of the OOS period
  end.oos <- ind.dates.oos[noos]
  end.is <- start.oos - 1

  # check that the first date index is greater than the max start index
  max.start <- max(start)
  if(start.is < max.start)
    stop("The start.date is too early!")

  # allocate variables
  ret <- rep(0,noos)
  in.stocks <- rep(FALSE,noos)
  rule.name <- rep("",noos)
  ma.name <- rep("",noos)
  param <- matrix(NA,nrow=3,ncol=noos)
  n <- ncol(actret) # this is the number of simulated strategies
  PM <- rep(0,n) # to hold the performance measures
  actret <- actret - rfret

  # main loop
  for(i in 1:noos) {
    # check if we need to refit
    if((i-1)%%refit.every==0) {
      # compute the performance measures in the IS period
      for(j in 1:n) PM[j] <- FUN(actret[start.is:end.is,j])
      ind <- which.max(PM) # index of the strategy with the best IS performance
    }
    # determine the active return
    in.stocks[i] <- instocks[end.is, ind]
    if(shorts) {
      if (i == 1) { # very first transaction, no transaction costs
        if (in.stocks[i]) ret[i]<-totret[end.is+1] else ret[i]<-2*rfret[end.is+1]-totret[end.is+1]
      } else { # every else transaction
        if (in.stocks[i]) {
          # hold in stocks
          if (in.stocks[i-1]) ret[i]<-totret[end.is+1] else ret[i]<-totret[end.is+1]-2*tc
        } else {
          # keep shorting
          if (in.stocks[i-1]) ret[i]<-2*rfret[end.is+1]-totret[end.is+1]-2*tc else ret[i]<-2*rfret[end.is+1]-totret[end.is+1]
        }
      }
    } else {
      if (i == 1) { # very first transaction, no transaction costs
        if (in.stocks[i]) ret[i]<-totret[end.is+1] else ret[i]<-rfret[end.is+1]
      } else { # every else transaction
        if (in.stocks[i]) {
          # hold in stocks
          if (in.stocks[i-1]) ret[i]<-totret[end.is+1] else ret[i]<-totret[end.is+1]-tc
        } else {
          # hold in cash
          if (in.stocks[i-1]) ret[i]<-rfret[end.is+1]-tc else ret[i]<-rfret[end.is+1]
        }
      }
    }

    end.is <- end.is + 1 # increase the end of the IS period
    if(rolling)  start.is <- start.is + 1 # increase the start of the IS period
    # add the strategy description
    rule.name[i] <- as.character(desc$Trading.Rule[ind])
    ma.name[i] <- as.character(desc$MA.Type[ind])
    param[1,i] <- desc$Param.1[ind]
    param[2,i] <- desc$Param.2[ind]
    param[3,i] <- desc$Param.3[ind]
  }

  desc <- data.frame(Trading.Rule=rule.name, MA.Type=ma.name,
                     Param.1 = param[1,], Param.2 = param[2,], Param.3 = param[3,])

  totret <- totret[start.oos:end.oos]
  rfret <- rfret[start.oos:end.oos]
  dates <- dates[start.oos:end.oos]

  # return a list with results
  result <- list(actret=ret, totret=totret, rfret=rfret, rolling=rolling, resname="OOS",
                 dates=dates, instocks=in.stocks, desc=desc, FUN=FUN, fun.name=fun.name,
                 start.date=start.date, end.is.date=end.is.date, end.oos.date=end.oos.date)
  return(result)
}


#' Plots the rolling outperformance of the OOS strategy
#'
#' This function plots the rolling outperformance of the out-of-sample trading strategy
#'
#' @param results the results returned by function \code{\link{forward.test}}
#' @param start.date the start of the first rolling period
#' @param split.date the end of the first rolling period
#' @param end.date the end of the total period
#' @return None
#' @note The argument \code{results} contains, among other variables, the dates vector that defines the historical period
#' over which the trading strategies are simulated. The outperformance is defined as
#' the difference between the performance of a trading strategy and the performance of
#' the corresponding passive strategy.
#' @usage oos.outperformance.plot(results, start.date, split.date, end.date)
#' @family plotting functions
#' @examples
#' \dontrun{
#' library(matiming)
#' library(zoo)
#'
#' data <- sp500.monthly
#' dates <- index(data)
#' data <- coredata(data)
#' capret <- data[,"CAP"]
#' totret <- data[,"TOT"]
#' rfret <- data[,"RF"]
#'
#' tc <- 0.0025
#' fast <- 1:9
#' slow <- 2:18
#'
#' res.mac <- sim.mac.strategy(totret=totret, rfret=rfret, dates=dates,
#'                             capret=capret, tc=tc, fast=fast, slow=slow, FUN=SMA)
#'
#' res.mom <- sim.mom.strategy(totret=totret, rfret=rfret, dates=dates,
#'                             capret=capret, tc=tc, winsize=slow)
#'
#' results <- sim.results.merge(res.mac, res.mom)
#'
#' start.date <- as.Date("1930-01-01", format="%Y-%m-%d")
#' end.is.date <- as.Date("1949-12-31", format="%Y-%m-%d")
#' end.oos.date <- as.Date("2015-12-31", format="%Y-%m-%d")
#'
#' res <- forward.test(results, start.date=start.date, end.is.date=end.is.date,
#'                     end.oos.date=end.oos.date)
#'
#' # plot the rolling outperformance
#' start.date <- as.Date("1950-01-01", format="%Y-%m-%d")
#' split.date <- as.Date("1954-12-31", format="%Y-%m-%d")
#' end.date <- as.Date("2015-12-31", format="%Y-%m-%d")
#'
#' oos.outperformance.plot(res, start.date=start.date, split.date=split.date,
#'                         end.date=end.date)
#' }
#' @export oos.outperformance.plot

oos.outperformance.plot <- function(results, start.date, split.date, end.date) {

  if(results$resname != "OOS")
    stop("results object must be produced by forward.test function!")

  totret <- results$totret
  actret <- results$actret
  rfret <- results$rfret
  dates <- results$dates
  #instocks <- results$instocks
  FUN <- results$FUN
  start.is.date <- results$start.is.date
  end.is.date <- results$end.is.date
  end.oos.date <- results$end.oos.date

  # check the dates boundaries
  if(start.date < end.is.date)
    stop("The 'start.date' is earlier than the 'end.is.date' !")
  if(end.date > end.oos.date)
    stop("The 'end.date' is later than the 'end.oos.date' !")

  # select the indices of the required observations
  date.ind <- dates >= start.date & dates <= split.date

  if(is.na(which(date.ind)[1]))
    stop("Something is wrong. The period from 'start' to 'split' has zero length!")

  # check that the first date index is greater than the max start index
  start.rolling <- which(date.ind)[1]

  dates.out <- dates > split.date & dates <= end.date
  ind.dates.out <- which(dates.out) # indices of the OOS period
  start.out <- ind.dates.out[1]
  if(is.na(start.out))
    stop("Something is wrong. The period from 'split' to 'end' has zero length!")
  nout <- length(ind.dates.out) # the length of the OUT period
  end.out <- ind.dates.out[nout]
  end.rolling <- start.out - 1

  N <- nout # the number of rolling windows

  # allocate variables
  dPM <- rep(0, N) # to hold the difference in performance measures
  actret <- actret - rfret
  totret <- totret - rfret

  # main loop to compute the rolling outperformance
  for(i in 1:N) {
    # compute the performance measures in the rolling IS period
    dPM[i] <- FUN(actret[start.rolling:end.rolling]) - FUN(totret[start.rolling:end.rolling])
    # move the rolling window
    start.rolling <- start.rolling + 1
    end.rolling <- end.rolling + 1
  }

  n.years <- round(as.numeric(difftime(split.date, start.date, units="weeks"))/52.25)

  dates <- dates[dates > split.date & dates <= end.date]
  data <- zoo(dPM, order.by=dates)

  plot(data, col="red", lwd=2, ylab="Outperformance",
       main=paste("Rolling ",n.years," - year outperformance",sep=""), xlab="")
  abline(h=0, lty=2)
}


#' Tests whether the moving average strategy outperforms its passive counterpart
#'
#' This function tests whether the moving average strategy outperforms its passive counterpart. All tests are based on using a bootstrap method.
#'
#' @param results the results returned by function \code{\link{forward.test}} or \code{\link{back.test}}
#' @param digits defines the number of digits after the decimal delimiter to use in reporting
#' the performance of the moving average and buy-and-hold strategies
#' @param to.annual a coefficient used to annualize the performance measure. For example, when
#' the performance is measured using either \code{\link{Sharpe}} or \code{\link{Sortino}},
#' use \code{sqrt(252)} and \code{sqrt(12)} when the returns are at the daily and monthly frequency respectively.
#' If the performance is measred by \code{\link{Excret}}, use \code{12}
#' @param type determines the type of the bootstrap method. The default type is "ordinary" (the method of Efron, 1979).
#' The two other types are "block" (the method of Kunsch, 1989) and "stationary" (the method of Politis and Romano, 1994)
#' @param bLen a scalar that specifies the block length in the block-bootstrap method
#' @param R a scalar that specifies the number of bootstrap replicates
#' @param automatic a logical variable that specifies whether to compute the optimal block length
#' for the block-bootstrap and stationary bootstrap
#' @return The p-value of the outperformance test.
#' @usage outperformance.test(results, digits=3, to.annual=sqrt(12),
#'                            type=c("ordinary", "block","stationary"),
#'                            bLen=5, R=1000, automatic=FALSE)
#' @note If \code{results} are produced by \code{\link{back.test}}, then this function
#'  tests whether the best strategy in the back test outperforms the buy-and-hold strategy. Note that in this case the p-value is not ajdusted for the data-mining bias.
#'  Otherwise, if \code{results} are produced by \code{\link{forward.test}}, then this function
#'  tests whether the out-of-sample strategy outperforms the buy-and-hold strategy.
#'
#'  Regardless of the performance measure used (denoted by \eqn{PM}),
#'  this function performs the tests of the following null hypothesis: \eqn{H_0: PM_{MA}\leq PM_{BH}},
#'  where \eqn{PM_{MA}} is the performance of the moving average strategy and
#'  \eqn{PM_{BH}} is the performance of the buy-and-hold strategy. In words, the null hypothesis
#'  is that the performance of the moving average strategy is not better than the performance
#'  of the buy-and-hold strategy.
#'
#'  All tests are based on non-parametric bootstrap methods. The three bootstrap methods that are implemented are: (1) the ordinary
#'  bootstrap method of Efron (1979), (2) the block-bootstrap method of Kunsch (1989), and (3) the stationary block-bootstrap
#'  method of Politis and Romano (1994). In a block-bootstrap method one needs to either specify manually the block length or
#'  to allow the function to automatically determine the optimal block length. The optimal
#'  block length is computed using the method described in Patton, Politis and White (2009). To determine
#'  the optimal block length, function \code{b.star} from package \code{np} is used (Hayfield and Racine (2008)).
#' @references
#' Efron, B., Bootstrap Methods: Another Look at the Jackknife. Annals of Statistics, 1979, 7, 1-26.
#'
#' Kunsch, H.R., The Jacknife and the Bootstrap for General Stationary Observations. Annals of Statistics, 1989, 17, 1217-1241.
#'
#' Politis, D. and Romano, J., The Stationary Bootstrap. Journal of the American Statistical Association, 1994, 89, 1303-1313.
#'
#' Politis, D.N. and H. White (2004), Automatic block-length selection for the dependent bootstrap, Econometric Reviews 23 (1), 53-70.
#'
#' Patton, A. and D.N. Politis and H. White (2009), CORRECTION TO: Automatic block-length selection for the dependent bootstrap by D. Politis and H. White, Econometric Reviews 28 (4), 372-375.
#'
#' Hayfield, T. and J. Racine (2008). Nonparametric Econometrics: The np Package. Journal of Statistical Software, 27 (5).
#' @family strategy testing functions
#' @examples
#' # Suppose that "results" are the results of simulations of different trading strategies
#' \dontrun{
#' start.date <- as.Date("1930-01-01", format="%Y-%m-%d")
#' end.is.date <- as.Date("1939-12-31", format="%Y-%m-%d")
#' end.oos.date <- as.Date("2015-12-31", format="%Y-%m-%d")
#' refit.every <- 5
#' include.bh <- TRUE
#' res <- forward.test(results, start.date=start.date, end.is.date=end.is.date,
#'     end.oos.date=end.oos.date, refit.every=refit.every, include.bh=include.bh, FUN=Sortino)
#' outperformance.test(res, type="stationary", automatic=TRUE)
#' }
#' @export outperformance.test

outperformance.test <- function(results, digits=3, to.annual=sqrt(12),
                                type=c("ordinary", "block", "stationary"),
                                bLen=5, R=1000, automatic=FALSE) {
# this function prints out the peformance of the passive and
# market timing strategies, plus test the null hypothesis
# H0=Performance.MA <= Performance.BH
# Inputs:
# results - results of oos simulation
# type   - the type of the method
# bLen   - the block length
# R      - the number of simulations
# automatic - whether to estimate automatically the block length

  if( (results$resname != "IS") & (results$resname != "OOS") )
    stop("results object must be produced by either forward.test or back.test function!")

  if(!is.logical(automatic))
    stop("Argument 'automatic' must be of logical type!")

  if(!is.numeric(bLen))
    stop("Something is wrong. Argument 'bLen' is not of a numerical type!")

  if(!is.numeric(R))
    stop("Something is wrong. Argument 'R' is not of a numerical type!")

  if(!is.numeric(to.annual))
    stop("Something is wrong. Argument 'to.annual' is not of a numerical type!")

  totret <- results$totret
  actret <- results$actret
  rfret <- results$rfret
  dates <- results$dates
  FUN <- results$FUN
  fun.name <- results$fun.name

  if(is.matrix(actret)) {
    actret <- actret[,1]
  }
  actret <- as.vector(actret)

  # print out the results
  etot <- totret-rfret
  eact <- actret-rfret
  cat("==============================================================\n\n")
  cat("Performance of the BH strategy =", outstr(FUN(etot)*to.annual,digits), "\n")
  cat("Performance of the MA strategy =", outstr(FUN(eact)*to.annual,digits), "\n")

  x <- cbind(etot,eact)
  if(automatic) {
    # compute the otpimal block length
    res <- b.star(x)
    bLen <- round(mean(res[,1]))
  }
  bLen <- max(1,bLen) # for robustness
  R <- max(1,R)
  dPM <- rep(0,R)

  if(length(type)>1) type <- "ordinary"
  type.str <- type
  if(type=="ordinary") bLen <- 1
  type <- match.arg(type)
  # main simulation loop
  for(i in 1:R) {
    switch(type,
           ordinary = { y <- simple_bootstrap(x)},
           block = { y <- block_bootstrap(x, bLen)},
           stationary = { y <- stat_bootstrap(x, bLen)}
    )
    dPM[i] <- FUN(y[,2]) - FUN(y[,1])
  }
  # compute the probability
  n <- length(which(dPM <= 0))
  pval <- n/R
  cat("\n==============================================================\n\n")
  cat("The following null hypothesis is tested:\n")
  cat("The performance of MA strategy <= the performance of BH strategy\n")
  cat("Bootstrap method:", type.str, "\n")
  cat("Block length:", bLen, "\n")
  cat("Number of simulations:", R, "\n")
  cat("P-value of the null hypotheis =", outstr(pval,digits), "\n\n")
  return(pval)
}


findDrawdowns <- function(index, n.dd=10) {
  # find the drawdowns
  # input
  # index - the index of stock prices

  drawdown.start <- vector() # the index of a top
  drawdown.end <- vector() # the index of subsequent bottom

  # find the beginning and end of each bull - bear periods
  n <- length(index)
  top <- 1 # current top
  for(i in 1:n) {
    if(i==1) {
      # assume that the first point is a top
      top <- i
      drawdown.start <- c(drawdown.start, i)
      next
    }
    if(index[i] > index[top]) {
      # this is a new top
      # find the minimum
      minval <- min(index[top:i])
      if(minval < index[top]) {
        # record the new bottom value
        ind.min <- which.min(index[top:i]) + top - 1
        # add end
        drawdown.end <- c(drawdown.end, ind.min)
        # mark the new start
        drawdown.start <- c(drawdown.start, i)
        #top <- i
        #next
      }
      top <- i
      drawdown.start[length(drawdown.start)] <- i
    }
    if(i==n) {
      # this is the end of the index
      if(top==n) {
        # new top is the last observation
        # delete the start
        drawdown.start <- drawdown.start[1:(length(drawdown.start)-1)]
      } else {
        # find the minimum
        minval <- min(index[top:i])
        if(minval < index[top]) {
          # record the new bottom value
          ind.min <- which.min(index[top:i]) + top - 1
          # add end
          drawdown.end <- c(drawdown.end, ind.min)
        } else {
          # delete the start
          drawdown.start <- drawdown.start[1:(length(drawdown.start)-1)]
        }
      }
    }
  }
  # final check for consistency
  nStarts <- length(drawdown.start)
  nEnds <- length(drawdown.end)
  if(nStarts!=nEnds)
    stop("Length of start indices is not equal to length of end indices!")
  #cat("Number of drawdowns =", nStarts, "\n")

  # compute the summary statistics
  drawdown <- rep(0,nStarts)
  for(i in 1:nStarts) {
    drawdown[i] <- (index[drawdown.start[i]] - index[drawdown.end[i]])/index[drawdown.start[i]]*100
    #cat("Drawdown i= ",i," amplitude=",drawdown[i],"\n")
  }

  mean.dd <- mean(drawdown)
  median.dd <- median(drawdown)
  max.dd <- max(drawdown)
  min.dd <- min(drawdown)
  dd.sorted <- sort(drawdown, decreasing = TRUE)
  if(nStarts > n.dd) {
    mean.max.dd <- mean(dd.sorted[1:n.dd])
  } else {
    mean.max.dd <- mean(dd.sorted[1:nStarts])
  }

  DD <- c(mean.dd, mean.max.dd, max.dd)
  return(DD)
}

runningDrawdown <- function(index) {
  # input
  # index - the index of stock prices
  n <- length(index)
  drawdown <- rep(0,n)
  for(i in 1:n) {
    if(i==1) {
      # assume that the first point is a top
      top <- i
      next
    }
    if(index[i] > index[top]) {
      top <- i
    } else {
      drawdown[i] <- (index[top] - index[i])/index[top]*100
    }
  }
  return(drawdown)
}

#' Computes the summary statistics of the moving average trading strategy and the corresponding buy-and-hold strategy
#'
#' This function computes the summary statistics of the moving average trading strategy and the corresponding buy-and-hold strategy
#'
#' @param results the results returned by function \code{\link{forward.test}} or \code{\link{back.test}}
#' @param nobs.a.year a scalar that defines the number of observations a year. For example, for monthly data
#' the number of observations a year equals 12
#' @param to.annual a coefficient used to annualize the performance measure. For example, when
#' the performance is measured using either \code{\link{Sharpe}} or \code{\link{Sortino}},
#' use \code{sqrt(252)} and \code{sqrt(12)} when the returns are at the daily
#' and monthly frequency respectively. If the performance is measred by \code{\link{Excret}}, use \code{12}
#' @param type determines the type of the bootstrap method. The default type is "ordinary" (the method of Efron, 1979).
#' The two other types are "block" (the method of Kunsch, 1989) and "stationary" (the method of Politis and Romano, 1994)
#' @param bLen a scalar that specifies the block length in the block-bootstrap method
#' @param R a scalar that specifies the number of bootstrap replicates
#' @param automatic a logical variable that specifies whether to compute the optimal block length
#' for the block-bootstrap and stationary bootstrap
#' @param plot.results a logical value indicating whether to plot the cumulative returns and drawdowns to
#' the moving average strategy and the buy-and-hold strategy
#' @return An object of class \code{data.frame} containing two columns: \code{BH} and \code{MA}.
#' The first column contains the summary statistics for the Buy-and-Hold strategy, the second
#' column contains the summary statistics for the Moving Average strategy.
#' Each column contains the following rows:
#' \describe{
#'   \item{Mean returns}{annualized mean returns}
#'   \item{Std. deviation}{annualized standard deviation}
#'   \item{Minimum return}{minimum return}
#'   \item{Maximum return}{maximum return}
#'   \item{Skewness}{skewness of return distribution}
#'   \item{Kurtosis}{kurtosis of return distribution}
#'   \item{Average drawdown}{Average drawdown}
#'   \item{Average max drawdown}{Average of the 10 largest drawdowns}
#'   \item{Maximum drawdown}{Maximum drawdown}
#'   \item{Performance}{Performance measure}
#'   \item{P-value}{P-value of the test of outperformance}
#'   \item{Rolling 5-year Win }{Probability that the moving average strategy outperforms its passive counterpart over a 5-year horizon}
#'   \item{Rolling 10-year Win }{Probability that the moving average strategy outperforms its passive counterpart over a 10-year horizon}
#' }
#' @usage descriptive.statistics(results, nobs.a.year = 12, to.annual=sqrt(12),
#'                               type=c("ordinary", "block", "stationary"),
#'                               bLen=5, R=1000, automatic=FALSE, plot.results=FALSE)
#' @note The length of the tested period should be longer than 10 years (this is needed to compute the Rolling 10-year Win).
#' P-value is the p-value of the outperformance test. This p-value is computed in the same
#' manner as in function \code{\link{outperformance.test}}.
#'  If \code{results} are produced by \code{\link{back.test}}, then this function
#'  computes the descriptive statistics and the p-value of the best strategy in the back test. In this case the p-value is not adjusted for the data-mining bias; hence,
#'  this p-value is unreliable. Otherwise, if \code{results} are produced by \code{\link{forward.test}}, then this function
#'  computes the descriptive statistics and the p-value of the out-of-sample strategy.
#' @family strategy summary functions
#' @examples
#' \dontrun{
#' library(matiming)
#' library(xtable)
#' library(zoo)
#'
#' data <- sp500.monthly
#' dates <- index(data)
#' data <- coredata(data)
#' capret <- data[,"CAP"]
#' totret <- data[,"TOT"]
#' rfret <- data[,"RF"]
#'
#' tc <- 0.0025
#' fast <- 1:9
#' slow <- 2:18
#'
#' res.mac <- sim.mac.strategy(totret=totret, rfret=rfret, dates=dates,
#'                          capret=capret, tc=tc, fast=fast, slow=slow, FUN=SMA)
#'
#' res.mom <- sim.mom.strategy(totret=totret, rfret=rfret, dates=dates,
#'                          capret=capret, tc=tc, winsize=slow)
#'
#' results <- sim.results.merge(res.mac, res.mom)
#'
#' start.date <- as.Date("1920-01-01", format="%Y-%m-%d")
#' end.is.date <- as.Date("1943-12-31", format="%Y-%m-%d")
#' end.oos.date <- as.Date("2015-12-31", format="%Y-%m-%d")
#' refit.every <- 1
#' rolling <- FALSE
#'
#' res <- forward.test(results, start.date, end.is.date, end.oos.date,
#'                          refit.every=refit.every, rolling=rolling, FUN=Sharpe)
#'
#' df <- descriptive.statistics(res, plot.results=TRUE)
#' xtab <- xtable(df, digits=2)
#' print(xtab)
#' }
#' @export descriptive.statistics

descriptive.statistics <- function(results, nobs.a.year = 12, to.annual=sqrt(12),
                               type=c("ordinary", "block", "stationary"),
                               bLen=5, R=1000, automatic=FALSE, plot.results=FALSE) {

  if( (results$resname != "IS") & (results$resname != "OOS") )
    stop("results object must be produced by either forward.test or back.test function!")

  if(!is.logical(automatic))
    stop("Argument 'automatic' must be of logical type!")

  if(!is.logical(plot.results))
    stop("Argument 'plot.results' must be of logical type!")

  if(!is.numeric(bLen))
    stop("Something is wrong. Argument 'bLen' is not of a numerical type!")

  if(!is.numeric(R))
    stop("Something is wrong. Argument 'R' is not of a numerical type!")

  if(!is.numeric(to.annual))
    stop("Something is wrong. Argument 'to.annual' is not of a numerical type!")

  if(!is.numeric(nobs.a.year))
    stop("Something is wrong. Argument 'annual.observations' is not of a numerical type!")


  totret <- results$totret
  actret <- results$actret
  rfret <- results$rfret
  dates <- results$dates
  FUN <- results$FUN
  fun.name <- results$fun.name

  if(nobs.a.year*10 >= length(dates) )
    stop("The tested period should be longer than 10 years!")

  if(is.matrix(actret)) {
    actret <- actret[,1]
  }
  actret <- as.vector(actret)

  rnames <- c("Mean returns %",
              "Std. deviation %",
              "Minimum return %",
              "Maximum return %",
              "Skewness",
              "Kurtosis",
              "Average drawdown %",
              "Average max drawdown %",
              "Maximum drawdown %",
              fun.name,
              "P-value",
              "Rolling 5-year Win %",
              "Rolling 10-year Win %")

  #====================================================
  # compute the summary statistics

  # Active strategy
  mean.ma <- mean(actret)*nobs.a.year*100
  std.ma <- sd(actret)*sqrt(nobs.a.year)*100
  skew.ma <- skewness(actret)
  kurt.ma <- kurtosis(actret)
  min.ma <- min(actret)*100
  max.ma <- max(actret)*100
  pm.ma <- FUN(actret-rfret)*to.annual

  # Passive strategy
  mean.bh <- mean(totret)*nobs.a.year*100
  std.bh <- sd(totret)*sqrt(nobs.a.year)*100
  skew.bh <- skewness(totret)
  kurt.bh <- kurtosis(totret)
  min.bh <- min(totret)*100
  max.bh <- max(totret)*100
  pm.bh <- FUN(totret-rfret)*to.annual

  #====================================================
  # compute the drawdowns
  ind.bh <- 10*cumprod(1+totret)
  dd.bh <- findDrawdowns(ind.bh)
  ind.ma <- 10*cumprod(1+actret)
  dd.ma <- findDrawdowns(ind.ma)

  #====================================================
  # perform the test of outperformance hypothesis

  etot <- totret-rfret
  eact <- actret-rfret

  x <- cbind(etot,eact)
  if(automatic) {
    # compute the otpimal block length
    res <- b.star(x)
    bLen <- round(mean(res[,1]))
  }
  bLen <- max(1,bLen) # for robustness
  R <- max(1,R)
  dPM <- rep(0,R)

  if(length(type)>1) type <- "ordinary"
  type.str <- type
  if(type=="ordinary") bLen <- 1
  type <- match.arg(type)
  # main simulation loop
  for(i in 1:R) {
    switch(type,
           ordinary = { y <- simple_bootstrap(x)},
           block = { y <- block_bootstrap(x, bLen)},
           stationary = { y <- stat_bootstrap(x, bLen)}
    )
    dPM[i] <- FUN(y[,2]) - FUN(y[,1])
  }
  # compute the probability
  n <- length(which(dPM <= 0))
  pval <- n/R

  #=========================================================
  # rolling outperformance
  nobs <- length(dates)

  # compute rolling 5-year win %
  win.len <- 5*nobs.a.year
  n <- nobs - win.len + 1
  dPM <- rep(0,n)
  for(i in 1:n)
    dPM[i] <- FUN(eact[i:(i+win.len-1)]) - FUN(etot[i:(i+win.len-1)])
  Rol.5.win <- length(which(dPM >= 0))/n*100

  # compute rolling 10-year win %
  win.len <- 10*nobs.a.year
  n <- nobs - win.len + 1
  dPM <- rep(0,n)
  for(i in 1:n)
    dPM[i] <- FUN(eact[i:(i+win.len-1)]) - FUN(etot[i:(i+win.len-1)])
  Rol.10.win <- length(which(dPM >= 0))/n*100

  #======================================================
  # construct the data frame with results
  MA <- c(mean.ma, std.ma, min.ma, max.ma, skew.ma, kurt.ma,
          dd.ma[1], dd.ma[2], dd.ma[3],
          pm.ma, pval, Rol.5.win, Rol.10.win)
  BH <- c(mean.bh, std.bh, min.bh, max.bh, skew.bh, kurt.bh,
          dd.bh[1], dd.bh[2], dd.bh[3],
          pm.bh, NA, NA, NA)
  df <- data.frame(BH=BH, MA=MA)
  row.names(df) <- rnames

  if(plot.results) {
    ind.pas <- 100*cumprod(1+totret)
    ind.act <- 100*cumprod(1+actret)

    dd.pas <- -runningDrawdown(ind.pas)
    dd.act <- -runningDrawdown(ind.act)

    sname1 <- "Buy and Hold"
    sname2 <- "Moving Average"

    old.par <- par(mfrow=c(2, 1), mar = c(2.5, 4, 0.5, 0.5))

    cumret <- cbind(ind.pas,ind.act)
    data <- zoo(log(cumret), order.by=dates)
    plot(data, plot.type="single", col=1:2, xlab="", ylab="Log cumulative return")
    legend("topleft", legend=c(sname1,sname2), col=c("black","red"), lty=1, bty = "n")

    dd <- cbind(dd.pas,dd.act)
    data <- zoo(dd, order.by=dates)
    plot(data, plot.type="single", col=1:2, xlab="", ylab="Drawdown, %")
    legend("bottomleft", legend=c(sname1,sname2), col=c("black","red"), lty=1, bty = "n")

    par(old.par)
  }

  return(df)
}

