#include <Rcpp.h>
using namespace Rcpp;

//' Computes the mean excess return
//'
//' This function computes the mean excess return. Excess return is the return in excess of the risk-free rate of return
//'
//' @param x the vector of excess returns
//' @return the mean excess return (a scalar)
//' @usage Excret(x)
//' @family performance measures
//' @export
// [[Rcpp::export]]

double Excret(NumericVector x) {
  return mean(x);
}

//' Computes the Sharpe ratio
//'
//' This function computes the Sharpe ratio
//'
//' @param x the vector of excess returns
//' @return the Sharpe ratio (a scalar)
//' @usage Sharpe(x)
//' @family performance measures
//' @references
//' Sharpe, W.F. (1994). The Sharpe Ratio. Journal of Portfolio Management, 21 (1), 49-58.
//' @export
// [[Rcpp::export]]

double Sharpe(NumericVector x) {
  return mean(x)/sd(x);
}

//' Computes the Sortino ratio
//'
//' This function computes the Sortino ratio
//'
//' @param x the vector of excess returns
//' @return the Sortino ratio (a scalar)
//' @usage Sortino(x)
//' @family performance measures
//' @references
//' Sortino, F.A. and Price, L.N. (1994). Performance measurement in a downside risk framework. Journal of Investing, 3 (3), 59-64.
//' @export
// [[Rcpp::export]]

double Sortino(NumericVector x) {
  NumericVector s = pmin(x, 0);
  return mean(x)/sqrt( mean(s*s) );
}

//' Computes the Simple Moving Average (SMA)
//'
//' This function computes the SMA of prices
//'
//' @param price the vector of prices
//' @param n the length of the averaging window (a scalar)
//' @return the SMA of prices (a vector)
//' @usage SMA(price, n)
//' @family moving average functions
//' @export
// [[Rcpp::export]]

NumericVector SMA(const NumericVector & price, const int n) {
  // this function computes the Simple Moving Average
  // of prices over the averaging window of size n
  // using the recursive algorithm
  const int nobs = price.size();
  NumericVector ma(nobs);
  for(int i = (n-1); i < nobs; i++) {
    if(i == (n-1)) {
      // SMA is computed for the first time
      ma[i] = 0;
      for(int j = 0; j < n; j++)
        ma[i] += price[i-n+1+j];
      ma[i] = ma[i]/n;
    } else ma[i] = ma[i-1] + (price[i] - price[i-n])/n; // recursive computation
  }
  return(ma);
}

//' Computes the Linear Moving Average (LMA)
//'
//' This function computes the LMA of prices
//'
//' @param price the vector of prices
//' @param n the length of the averaging window (a scalar)
//' @return the LMA of prices (a vector)
//' @usage LMA(price, n)
//' @family moving average functions
//' @export
// [[Rcpp::export]]

NumericVector LMA(const NumericVector & price, const int n) {
  // this function computes the Linear Moving Average
  // of prices over the averaging window of size n
  // using the recursive algorithm
  const int nobs = price.size();
  int i=0;
  double sum = double(n)*(double(n)+1)/2; // the sum of the weights
  double total = 0; // to hold the total sum of prices in the rolling window
  // compute the LMA
  NumericVector ma(nobs);
  for(i = (n-1); i < nobs; i++) {
    if(i == (n-1)) {
      // LMA is computed for the first time
      ma[i] = 0;
      for(int j = 0; j < n; j++)  {
        ma[i] += (j+1)*price[i-n+1+j];
        total += price[i-n+1+j];
      }
      ma[i] = ma[i] / sum;
    } else {
      // recursive computation
      ma[i] = ma[i-1] + (double(n)*price[i]-total)/sum;
      total = total + price[i] - price[i-n];
    }
  }
  return(ma);
}

//' Computes the Exponential Moving Average (EMA)
//'
//' This function computes the infinite EMA of prices
//'
//' @param price the vector of prices
//' @param n the length of the averaging window (a scalar.
//' @note By definition, \eqn{n} is the length of the averaging window in SMA that has the same average lag time as EMA
//' @return the EMA of prices (a vector)
//' @usage EMA(price, n)
//' @family moving average functions
//' @export
// [[Rcpp::export]]

NumericVector EMA(const NumericVector & price, const int n) {
  // this function computes the Exponential Moving Average or prices
  // using the recursive algorithm
  const int nobs = price.size();
  const double lambda = 1 - 2/(double(n)+1);
  NumericVector ma(nobs);
  ma[0] = price[0];
  for(int i = 1; i < nobs; i++) ma[i] = (1-lambda)*price[i] + lambda*ma[i-1];
  return(ma);
}

// [[Rcpp::export]]

IntegerVector mae_signal(const NumericVector & price,
                         const NumericVector & lower,
                         const NumericVector & upper) {
  // this function computes the Momentum
  const int nobs = price.size();
  IntegerVector signal(nobs, 1);
  for(int i = 1; i < nobs; i++) {
    if(signal[i-1]==1) { // previous signal is BUY
      if(price[i] < lower[i]) signal[i] = -1; // change to SELL
    } else { // previous signal is SELL
      if(price[i] <= upper[i]) signal[i] = -1; // continue SELL
    }
  }
  return(signal);
}


// [[Rcpp::export]]

NumericVector MOM(const NumericVector & price, const int n) {
  // this function computes the Momentum
  const int nobs = price.size();
  NumericVector mom(nobs);
  for(int i = (n-1); i < nobs; i++) mom[i] = price[i] - price[i-n+1];
  return(mom);
}

// [[Rcpp::export]]
NumericVector sim_strategy(const NumericVector & totret,
                           const NumericVector & rfret,
                           const LogicalVector & buy_signal,
                           const double tc,
                           const bool shorts,
                           const int start)
{
  const int nobs = totret.size();
  NumericVector ret(nobs);
  for (int i=(start-1); i < (nobs-1); i++) {
  // simulate returns to the active strategy
    if(shorts) {
      // sell short when the signal is Sell
      if (i == (start-1)) {
        if (buy_signal[i]) {
          // first trading without transaction costs
          ret[i+1] = totret[i+1]; // buy stocks
        } else ret[i+1] = 2.0*rfret[i+1] - totret[i+1]; // sell short stocks
      } else {
        if (buy_signal[i]) {
          // hold in stocks
          if (buy_signal[i-1]) ret[i+1] = totret[i+1]; else ret[i+1] = totret[i+1] - 2.0*tc;
        } else { // buy == false
          // keep short position
          if (buy_signal[i-1]) ret[i+1] = 2.0*rfret[i+1] - 2.0*tc - totret[i+1]; else ret[i+1] = 2.0*rfret[i+1] - totret[i+1];
        }
      }
    } else {
      // move to cash when the signal is Sell
      if (i == (start-1)) {
        if (buy_signal[i]) {
          // first trading without transaction costs
          ret[i+1] = totret[i+1]; // hold in stocks
        } else ret[i+1] = rfret[i+1]; // hold in cash
      } else {
        if (buy_signal[i]) {
          // hold in stocks
          if (buy_signal[i-1]) ret[i+1] = totret[i+1]; else ret[i+1] = totret[i+1] - tc;
        } else {
          // hold in cash
          if (buy_signal[i-1]) ret[i+1] = rfret[i+1] - tc; else ret[i+1] = rfret[i+1];
        }
      }
    }
  }
  return(ret);
}

// [[Rcpp::export]]
NumericMatrix stat_bootstrap(NumericMatrix & x, const int bLen) {
  // function implements the stationary bootstrap of Politis and Romano 1994
  // Inputs:
  // x - nobs x 2 matrix of 2 dependent variables
  // bLen - average block length in the block bootstrap

  const int nobs = x.nrow();
  NumericMatrix y(nobs, 2);
  const double p = 1/double(bLen); // probability in the geometric distribution
  // generate start indices from uniform distribution
  IntegerVector start_indices = as<IntegerVector>(Rcpp::runif(nobs,0,nobs));
  // generate block lengths from geometric distribution
  IntegerVector block_lengths = as<IntegerVector>(Rcpp::rgeom(nobs,p)) + 1;
  // create variables
  int counter = 0;
  int start_ind = 0;
  int block_len_ind = 0;
  // main loop
  while(counter < nobs) {
    // select the current block of data
    int ind = start_indices[start_ind];
    int len = block_lengths[block_len_ind];
    for(int i = 0; i < len; i++) {
      y(counter,0) = x(ind,0);
      y(counter,1) = x(ind,1);
      ind += 1; // increase the current index
      if(ind >= nobs) ind = 0; // start from the beginning
      counter += 1; // increase the counter
      if(counter >= nobs) break; // end the for-loop
    }
    // to start a new block generation
    start_ind += 1;
    block_len_ind += 1;
  }
  return(y);
}

// [[Rcpp::export]]
NumericMatrix block_bootstrap(NumericMatrix & x, const int bLen) {
  // function implements the block bootstrap
  // Inputs:
  // x - nobs x 2 matrix of 2 dependent variables
  // bLen - the block length in the block bootstrap

  const int nobs = x.nrow();
  NumericMatrix y(nobs, 2);
  // generate start indices from uniform distribution
  IntegerVector start_indices = as<IntegerVector>(Rcpp::runif(nobs,1,nobs+1));
  // create variables
  int counter = 0;
  int start_ind = 0;
  // main loop
  while(counter < nobs) {
    // select the current block of data
    int ind = start_indices[start_ind];
    for(int i = 0; i < bLen; i++) {
      y(counter,0) = x(ind,0);
      y(counter,1) = x(ind,1);
      ind += 1; // increase the current index
      if(ind >= nobs) ind = 0; // start from the beginning
      counter += 1; // increase the counter
      if(counter >= nobs) break; // end the for-loop
    }
    // to start a new block generation
    start_ind += 1;
  }
  return(y);
}

// [[Rcpp::export]]
NumericMatrix simple_bootstrap(NumericMatrix & x) {
  // function implements the simple bootstrap
  // Inputs:
  // x - nobs x 2 matrix of 2 dependent variables

  const int nobs = x.nrow();
  NumericMatrix y(nobs, 2);
  // generate indices from uniform distribution
  IntegerVector ind = as<IntegerVector>(Rcpp::runif(nobs,0,nobs));
  for(int i = 0; i < nobs; i++) {
    y(i,0) = x(ind[i],0);
    y(i,1) = x(ind[i],1);
  }
  return(y);
}

//' @useDynLib matiming


